#ifndef AW9523_h
#define AW9523_h
#include <Arduino.h>
#include <Wire.h>

// aw9523 data register from datasheet
#define inputPort0 0x00     // P0 口输入状态
#define inputPort1 0x01     // P1 口输入状态

#define outputPort0 0x02    // P0 口输出状态
#define outputPort1 0x03    // P1 口输出状态

#define configPort0 0x04    // P0 口输入输出配置
#define configPort1 0x05    // P1 口输入输出配置

#define CTL 0x11            // 全局控制 工作模式切换

#define LEDModePort0 0x12   // P0 口 工作模式切换
#define LEDModePort1 0x13   // P1 口 工作模式切换

class AW9523 {

private:
  int modifyBit(int currentByte, int position, int bit)  // bit field, change position, change value
  {
    int mask = 1 << position;
    // Serial.print("currentByte ");
    // Serial.println(currentByte);
    // Serial.print("mask ");
    // Serial.println(mask);
    // Serial.print("bit ");
    // Serial.println(bit);
    return (currentByte & ~mask) | ((bit << position) & mask);
  }

  int i2cAddress = 0x58;

  int pinModeP0 = 0xff;  // all as input (1= input)
  int pinModeP1 = 0xff;  // all as input (1= input)
  int pinDataP0 = 0x00;  // all as 0 default
  int pinDataP1 = 0x00;  // all as 0 default

public:
  AW9523() {
    delay(6);
  }

  // device address 0x5B ~ 0x58
  void init(int i2cAdd) {
    i2cAddress = i2cAdd;                 // 配置地址

#if defined(NRF5) || defined(NRF52833)
    Wire.begin();
#endif

    Wire.beginTransmission(i2cAddress);   // transmit to device
    Wire.write(CTL);                      // CTL
    Wire.write(0x10);                     // data , P0口驱动模式？
    Wire.endTransmission();               // stop transmitting
    
    // 配置默认使用GPIO模式
    Wire.beginTransmission(i2cAddress);   // transmit to device
    Wire.write(LEDModePort0);             // led mode register p0
    Wire.write(0xff);                     // data , all pin as GPIO模式
    Wire.endTransmission();               // stop transmitting

    Wire.beginTransmission(i2cAddress);   // transmit to device
    Wire.write(LEDModePort1);             // led mode register p1
    Wire.write(0xff);                     // data , all pin as GPIO模式
    Wire.endTransmission();               // stop transmitting

    // config config_port registers for all pins as input in initilize mode
    /*  04 config p1
        05 config p2   */
    Wire.beginTransmission(i2cAddress);  // transmit to device
    Wire.write(configPort0);             // config register p0
    Wire.write(pinModeP0);               // data , all pin as input
    Wire.endTransmission();              // stop transmitting

    Wire.beginTransmission(i2cAddress);  // transmit to device
    Wire.write(configPort1);             // config register p1
    Wire.write(pinModeP1);               // data , all pin as input
    Wire.endTransmission();              // stop transmitting
  }

  /*
    po_0 = 0      p1_0 = 8
    po_1   1      p1_1   9
    po_2   2      p1_2   10
    po_3   3      p1_3   11
    po_4   4      p1_4   12
    po_5   5      p1_5   13
    po_6   6      p1_6   14
    po_7   7      p1_7   15
  */
  // pin , CpinMode  INPUT == 0
  void pinMode(uint8_t pin, uint8_t mode) {
    if (pin < 8) {
      pinModeP0 = modifyBit(pinModeP0, pin, !mode);  // change relavent bit in pin mode
      Wire.beginTransmission(i2cAddress);  // transmit to device
      Wire.write(configPort0);             // config register p0
      Wire.write(pinModeP0);               // data
      Wire.endTransmission();              // stop transmitting

    } else {
      pinModeP1 = modifyBit(pinModeP1, pin - 8, !mode);  // change relavent bit in pin mode
      Wire.beginTransmission(i2cAddress);  // transmit to device
      Wire.write(configPort1);             // config register p1
      Wire.write(pinModeP1);               // data
      Wire.endTransmission();              // stop transmitting
    }
  }

  // HIGH ==1
  void digitalWrite(uint8_t pin, uint8_t mode) {
    if (pin < 8) {                                  // for 02H register
      pinDataP0 = modifyBit(pinDataP0, pin, mode);  // change relavent bit in pin mode
      Wire.beginTransmission(i2cAddress);  // transmit to device
      Wire.write(outputPort0);             // config register p0
      Wire.write(pinDataP0);               // data
      Wire.endTransmission();              // stop transmitting

    } else {                                            // for 03H register
      pinModeP1 = modifyBit(pinModeP1, pin - 8, mode);  // change relavent bit in pin mode
      Wire.beginTransmission(i2cAddress);  // transmit to device
      Wire.write(outputPort1);             // config register p1
      Wire.write(pinModeP1);               // data
      Wire.endTransmission();              // stop transmitting
    }
  }

  void analogWrite(uint8_t pin, uint8_t value) {
    uint8_t analogRegister = pin + 20;  // dim channel start form 20 t0 35

    Wire.beginTransmission(i2cAddress);  // transmit to device
    Wire.write(analogRegister);          // config register
    Wire.write(value);                   // data
    Wire.endTransmission();              // stop transmitting
  }

  bool digitalRead(uint8_t pin) {
    if (pin < 8) {  //00h reg
      Wire.beginTransmission(i2cAddress);  // transmit to device
      Wire.write(inputPort0);              // config input register
      Wire.endTransmission();              // stop transmitting

      Wire.beginTransmission(i2cAddress);  // transmit to device for read
      Wire.requestFrom(i2cAddress, 1);     // read 1 byte
      uint8_t ReadDataP0 = Wire.read();    // receive byte

      return (ReadDataP0 >> pin) & 1;
    } else {  // 01h reg
      Wire.beginTransmission(i2cAddress);  // transmit to device
      Wire.write(inputPort1);              // config input register
      Wire.endTransmission();              // stop transmitting

      Wire.beginTransmission(i2cAddress);  // transmit to device for read
      Wire.requestFrom(i2cAddress, 1);     // read 1 byte
      uint8_t ReadDataP1 = Wire.read();    // receive byte

      return (ReadDataP1 >> pin - 8) & 1;
    }
  }
};


#endif