/** 
 * @file yfrobot
 * @brief YFROBOT's sensors Mind+ library.
 * @n This is a MindPlus graphics programming extension for YFROBOT's module.
 * 
 * @copyright    YFROBOT,2022
 * @copyright    MIT Lesser General Public License
 * 
 * @author [email](yfrobot@qq.com)
 * @date  2022-02-22
*/

enum ADDR {
    //% block="0x58"
    0x58,
    //% block="0x59"
    0x59,
    //% block="0x5A"
    0x5A,
    //% block="0x5B"
    0x5B
}

enum ODMONOFF {
    //% block="LOW"
    LOW,
    //% block="HIGH"
    HIGH
}

enum I2CIOPin {
    //% blockId="PIN0" block="0"
    0,
    //% blockId="PIN1" block="1"
    1,
    //% blockId="PIN2" block="2"
    2,
    //% blockId="PIN3" block="3"
    3,
    //% blockId="PIN4" block="4"
    4,
    //% blockId="PIN5" block="5"
    5,
    //% blockId="PIN6" block="6"
    6,
    //% blockId="PIN7" block="7"
    7,
    //% blockId="PIN8" block="8"
    8,
    //% blockId="PIN9" block="9"
    9,
    //% blockId="PIN10" block="10"
    10,
    //% blockId="PIN11" block="11"
    11,
    //% blockId="PIN12" block="12"
    12,
    //% blockId="PIN13" block="13"
    13,
    //% blockId="PIN14" block="14"
    14,
    //% blockId="PIN15" block="15"
    15
}

//% color="#96c24e" iconWidth=50 iconHeight=40 芽绿
namespace i2cioext {

    //% block="set[I2CADDR] [PIN] output [ODMSTATE]" blockType="command"
    //% I2CADDR.shadow="dropdown" I2CADDR.options="ADDR" I2CADDR.defl="ADDR.0x58"
    //% PIN.shadow="dropdown" PIN.options="I2CIOPin"  PIN.defl="I2CIOPin.0"
    //% ODMSTATE.shadow="dropdown" ODMSTATE.options="ODMONOFF" ODMSTATE.defl="HIGH"
    export function digitalWrite(parameter: any, block: any) {
        let i2c_addr = parameter.I2CADDR.code;
        let opin = parameter.PIN.code;
        let ostate = parameter.ODMSTATE.code;
        if(Generator.board === 'microbit') {     //如果是microbit，生成如下代码
            Generator.addInclude(`include_aw9523`, `#include "YF_MicroBit_AW9523.h"`);
            Generator.addObject(`aw9523_${i2c_addr}`, `AW9523`, `aw_${i2c_addr};`);
            Generator.addSetup(`aw9523_${i2c_addr}.begin`, `aw_${i2c_addr}.init(${i2c_addr});`);

            Generator.addSetup(`aw9523_${i2c_addr}_${opin}.pinMode`, `aw_${i2c_addr}.pinMode(${opin}, OUTPUT);`);
            Generator.addCode(`aw_${i2c_addr}.digitalWrite(${opin},${ostate});`);

        } else { 
            Generator.addInclude(`include_aw9523`, `#include <Adafruit_AW9523.h>`);
            Generator.addObject(`aw9523_${i2c_addr}`, `Adafruit_AW9523`, `aw_${i2c_addr};`);
            Generator.addSetup(`aw9523_${i2c_addr}.begin`, `aw_${i2c_addr}.begin(${i2c_addr});`);

            Generator.addSetup(`aw9523_${i2c_addr}_${opin}.pinMode`, `aw_${i2c_addr}.pinMode(${opin}, OUTPUT);`);
            Generator.addCode(`aw_${i2c_addr}.digitalWrite(${opin},${ostate});`);
        }
    }

    //% block="read [I2CADDR] [PIN]" blockType="boolean"
    //% I2CADDR.shadow="dropdown" I2CADDR.options="ADDR" I2CADDR.defl="ADDR.0x58"
    //% PIN.shadow="dropdown" PIN.options="I2CIOPin"  PIN.defl="I2CIOPin.0" 
    export function digitalRead(parameter: any, block: any) {
        let i2c_addr = parameter.I2CADDR.code;
        let opin = parameter.PIN.code;
        
        if(Generator.board === 'microbit') {     //如果是microbit，生成如下代码
            Generator.addInclude(`include_aw9523`, `#include "YF_MicroBit_AW9523.h"`);
            Generator.addObject(`aw9523_${i2c_addr}`, `AW9523`, `aw_${i2c_addr};`);
            Generator.addSetup(`aw9523_${i2c_addr}.begin`, `aw_${i2c_addr}.init(${i2c_addr});`);

            Generator.addSetup(`aw9523_${i2c_addr}_${opin}.pinMode`, `aw_${i2c_addr}.pinMode(${opin}, INPUT);`);
            // 模块未引出中断引脚，无中断功能； 理论功能：设置中断后，引脚状态变化，则触发中断，中断引脚电平拉低。
            // Generator.addSetup(`aw9523_${i2c_addr}_${opin}.enableInterrupt`, `aw_${i2c_addr}.enableInterrupt(${opin}, true);`);
            Generator.addCode(`aw_${i2c_addr}.digitalRead(${opin})`);

        } else { 
            Generator.addInclude(`include_aw9523`, `#include <Adafruit_AW9523.h>`);
            Generator.addObject(`aw9523_${i2c_addr}`, `Adafruit_AW9523`, `aw_${i2c_addr};`);
            Generator.addSetup(`aw9523_${i2c_addr}.begin`, `aw_${i2c_addr}.begin(${i2c_addr});`);

            Generator.addSetup(`aw9523_${i2c_addr}_${opin}.pinMode`, `aw_${i2c_addr}.pinMode(${opin}, INPUT);`);
            // 模块未引出中断引脚，无中断功能； 理论功能：设置中断后，引脚状态变化，则触发中断，中断引脚电平拉低。
            // Generator.addSetup(`aw9523_${i2c_addr}_${opin}.enableInterrupt`, `aw_${i2c_addr}.enableInterrupt(${opin}, true);`);
            Generator.addCode(`aw_${i2c_addr}.digitalRead(${opin})`);
        }

    }
}
